@@ -125,7 +125,7 @@ def task_id_str(task_family, params):
     # task_id is a concatenation of task family, the first values of the first 3 parameters
     # sorted by parameter name and a md5hash of the family/parameters as a cananocalised json.
     param_str = json.dumps(params, separators=(',', ':'), sort_keys=True)
-    param_hash = hashlib.md5(param_str.encode('utf-8')).hexdigest()
+    param_hash = hashlib.new('md5', param_str.encode('utf-8'), usedforsecurity=False).hexdigest()
 
     param_summary = '_'.join(p[:TASK_ID_TRUNCATE_PARAMS]
                              for p in (params[p] for p in sorted(params)[:TASK_ID_INCLUDE_PARAMS]))
@@ -304,7 +304,7 @@ class Task(metaclass=Register):
 
     task_namespace = __not_user_specified
     """
-    This value can be overriden to set the namespace that will be used.
+    This value can be overridden to set the namespace that will be used.
     (See :ref:`Task.namespaces_famlies_and_ids`)
     If it's not specified and you try to read this value anyway, it will return
     garbage. Please use :py:meth:`get_task_namespace` to read the namespace.
@@ -765,6 +765,87 @@ class MixinNaiveBulkComplete:
         return generated_tuples
 
 
+class DynamicRequirements(object):
+    """
+    Wraps dynamic requirements yielded in tasks's run methods to control how completeness checks of
+    (e.g.) large chunks of tasks are performed. Besides the wrapped *requirements*, instances of
+    this class can be passed an optional function *custom_complete* that might implement an
+    optimized check for completeness. If set, the function will be called with a single argument,
+    *complete_fn*, which should be used to perform the per-task check. Example:
+
+    .. code-block:: python
+
+        class SomeTaskWithDynamicRequirements(luigi.Task):
+            ...
+
+            def run(self):
+                large_chunk_of_tasks = [OtherTask(i=i) for i in range(10000)]
+
+                def custom_complete(complete_fn):
+                    # example: assume OtherTask always write into the same directory, so just check
+                    #          if the first task is complete, and compare basenames for the rest
+                    if not complete_fn(large_chunk_of_tasks[0]):
+                        return False
+                    paths = [task.output().path for task in large_chunk_of_tasks]
+                    basenames = os.listdir(os.path.dirname(paths[0]))  # a single fs call
+                    return all(os.path.basename(path) in basenames for path in paths)
+
+                yield DynamicRequirements(large_chunk_of_tasks, custom_complete)
+
+    .. py:attribute:: requirements
+
+        The original, wrapped requirements.
+
+    .. py:attribute:: flat_requirements
+
+        Flattened view of the wrapped requirements (via :py:func:`flatten`). Read only.
+
+    .. py:attribute:: paths
+
+        Outputs of the requirements in the identical structure (via :py:func:`getpaths`). Read only.
+
+    .. py:attribute:: custom_complete
+
+       The optional, custom function performing the completeness check of the wrapped requirements.
+    """
+
+    def __init__(self, requirements, custom_complete=None):
+        super().__init__()
+
+        # store attributes
+        self.requirements = requirements
+        self.custom_complete = custom_complete
+
+        # cached flat requirements and paths
+        self._flat_requirements = None
+        self._paths = None
+
+    @property
+    def flat_requirements(self):
+        if self._flat_requirements is None:
+            self._flat_requirements = flatten(self.requirements)
+        return self._flat_requirements
+
+    @property
+    def paths(self):
+        if self._paths is None:
+            self._paths = getpaths(self.requirements)
+        return self._paths
+
+    def complete(self, complete_fn=None):
+        # default completeness check
+        if complete_fn is None:
+            def complete_fn(task):
+                return task.complete()
+
+        # use the custom complete function when set
+        if self.custom_complete:
+            return self.custom_complete(complete_fn)
+
+        # default implementation
+        return all(complete_fn(t) for t in self.flat_requirements)
+
+
 class ExternalTask(Task):
     """
     Subclass for references to external dependencies.
